# Aliases

## Variables de ambiente

VENV=.venv
LENGTH=79
FONT_DIR=$HOME/.fonts
ZSH_CUSTOM=$HOME/.oh-my-zsh.d
OZSH_ALIAS=$ZSH_CUSTOM/aliases.zsh
BASH_ALIAS=$HOME/.bash_aliases
RLOC_ALIAS=$HOME/Repositorios/aliases/aliases.sh
REPO_ALIAS=https://gitlab.com/perritotuerto/codigo/aliases/-/raw/no-masters/aliases.sh

## Eliminación de alias previos

unalias -a

## Atajos

alias .1='cd ..'
alias .2='cd ../..'
alias .3='cd ../../..'
alias c='bat --paging never'
alias e='v'
alias g='geany'
alias l='lsd -al --permission octal'
alias o='okular'
alias s='scribus'
alias t='tree'
alias v='vim'
alias cpd='docker cp'
alias cdr='cd ~/Repositori*'
alias docs='cd ~/Document*'
alias repos='cdr'
alias cloud='nube'
alias descargas='cdd'
alias documentos='docs'
alias permisos='stat -c "Permiso %a | Creación %w | Modificación %y | %n" *'
alias stats='zsh_stats'

## Atajos SSH

alias adjetiva='ssh adjetiva'
alias agroteso='ssh agroteso'
alias aml='ssh aml'
alias aml-stats='ssh aml "docker stats -a"'
alias cel='ssh cel'
alias clientes='ssh clientes'
alias cuates='ssh cuates'
alias de='ssh -X de'
alias pc='ssh -X pc'
alias perro='ssh -X perro'
alias ppp='ssh ppp'
alias prolibreros='ssh prolibreros'
alias vpn='ssh vpn'
alias vpn-stats='ssh vpn "docker stats -a"'

## Personalizaciones

alias mv='mv -v'
alias rm='rm -v'
alias cp='cp -rv'
alias df='df -hT'
alias du='du -sh'
alias sed='sed -zr'
alias ssh='ssh -o ServerAliveInterval=30'
alias free='free -h'
alias diff='diff --color'
alias mkdir='mkdir -p'
alias rsync='rsync -varuchz --progress'
alias rdiff-backup='rdiff-backup -v 5'
alias perl-rename='perl-rename -v'
alias pip='pip3'
alias python='python3'
alias ipython='ipython3'
alias selene="selene --config $HOME/.config/selene/config.toml"

## Acciones

alias buscar="grep -REin --color=always --exclude-dir={.git,.venv,.pytest_cache,__pycache__,\*egg-info}"
alias search="buscar"
alias rastrear='fd'
alias trace='rastrear'
alias rastrear-todo='find / -iname'
alias trace-all='rastrear-todo'
alias copiar='xclip -selection clipboard <'
alias copy='copiar'
alias refresh='recargar'
alias install='instalar'
alias uninstall='desinstalar'
alias remove='desinstalar'
alias update='actualizar'
alias upgrade='actualizar'

## A modo de programitas

alias alv='rm -rf'
alias doc2md='pandoc --markdown-headings=atx -V lang=es'
alias mi-ip='curl https://ipapi.co/json'
alias my-ip='mi-ip'
alias pecas-legacy="sudo docker start pecas && sudo docker exec -it pecas /bin/zsh -l"
alias pacman-limpiar='echo "Limpiando caché…" && sudo pacman -Scc && echo "\nEliminando paquetes huérfanos…" && sudo pacman -Rs $(pacman -Qqdt)'
alias pacman-clean='pacman-limpiar'
alias docker-limpiar='sudo docker system prune -a'
alias docker-clean='docker-limpiar'
alias añadir-fuentes='font-add'
alias tiempo='/usr/bin/t'
alias untar='tar -xvf'
alias untargz='tar -xvzf'

## Monerías

alias arcoiris='journalctl --no-pager | dotacat'
alias matrix='tmatrix --mode=dense -f 0,5 -C magenta'
alias acuario='asciiquarium'
alias tren='sl'
alias fantasma='oneko &'

## Funciones

### Detecta tipo de distribución

distro-name () {
  echo $(awk -F= '/^NAME/{print $2}' /etc/os-release) | sed 's/"//g'
}

### Imprime errores

echo-error () {
  echo "ERROR: $1"
  return 1
}

### Actualizador

actualizar () {
  case $1 in
  alias | aliases)
    echo "Actualizando alias"
    curl -o $BASH_ALIAS $REPO_ALIAS
    if [ -f "$OZSH_ALIAS" ]; then
      cp $BASH_ALIAS $OZSH_ALIAS
    fi
    recargar
    ;;
  paquetes | packages)
    echo "Actualizando paquetes"
    if [[ $(distro-name) = "Arch Linux" ]]; then
      sudo pacman -Syy
      sudo pacman -S archlinux-keyring
    else
      sudo apt update
    fi
    ;;
  todo | sistema | all | system)
    echo "Actualizando sistema"
    if [[ $(distro-name) = "Arch Linux" ]]; then
      yay -Su
    else
      sudo apt upgrade
    fi
    ;;
  -h | --help | help | ayuda)
    actualizar-help ;;
  *)
    actualizar alias
    actualizar paquetes
    actualizar sistema
    ;;
  esac
}

actualizar-help () {
  echo "actualizar: utilidad para hacer actualizaciones.

Uso:
  actualizar [OPT]

Opciones:
  alias | aliases                Actualiza alias.
  paquetes | packages            Actualiza repositorio de paquetes.
  -h | --help | help | ayuda     Despliega esta ayuda.
  todo | sistema | all | system  Actualiza el sistema.
  vacío                          Actualiza el sistema con actualización previa de paquetes.

Ejemplos:
  actualizar alias
  actualizar paquetes
  actualizar -h
  actualizar todo
  actualizar"
}

### Instalador

instalar () {
  if [[ $(distro-name) = "Arch Linux" ]]; then
    yay -Syy $@
  else
    sudo apt install $@
  fi
}

### Desinstalador

desinstalar () {
  if [[ $(distro-name) = "Arch Linux" ]]; then
    yay -Rss $@
  else
    sudo apt purge $@
    sudo apt clean
    sudo apt autoremove
  fi
}

### Gestión del home

home () {
  if [ -d "$HOME/.git" ]; then
    case $1 in
    estado | status)
      git -C $HOME status
      ;;
    editar | edit)
      e $HOME/.gitignore
      ;;
    guardar | save)
      git -C $HOME add .
      git -C $HOME commit -m "Actualización"
      ;;
    empujar | push)
      if output=$(git -C $HOME status --porcelain) && [ ! -z "$output" ]; then
        home guardar
      fi
      unset output
      git -C $HOME pushall
      ;;
    -h | --help | help | ayuda)
      home-help ;;
    *)
      if [ -z "$1" ]; then
        echo-error "al menos un argumento es necesario; por ejemplo 'home -h'"
      else
        echo-error "comando '$1' inválido"
      fi
      ;;
    esac
  else
    echo-error "el home '$HOME' no es un directorio git"
  fi
}

home-help () {
  echo "home: utilidad para gestionar el home.

Home: $HOME
Estado: $(if [ -d "$HOME/.git" ]; then echo "OK"; else echo "requiere ser un repositorio git"; fi)

Uso:
  home [OPT]

Opciones:
  estado | status             Muestra el estado del home.
  editar | edit               Edita el archivo .gitignore del home.
  guardar | save              Guarda el estado del home.
  empujar | push              Empuja el estado del home al repositorio.
  -h | --help | help | ayuda  Despliega esta ayuda.

Ejemplos:
  home estado
  home editar
  home guardar
  home empujar
  home -h"
}

### Gestión de alias

aliases () {
  case $1 in
  actualizar | update)
    actualizar aliases
    ;;
  ver | see)
    if [ -f "$OZSH_ALIAS" ]; then
      echo $OZSH_ALIAS
      c $OZSH_ALIAS
    else
      echo $BASH_ALIAS
      c $BASH_ALIAS
    fi
    ;;
  editar | edit)
    if [ -f "$OZSH_ALIAS" ]; then
      e $OZSH_ALIAS
    else
      e $BASH_ALIAS
    fi
    ;;
  empujar | push)
    repo=$(parent $RLOC_ALIAS)
    if [ -f "$RLOC_ALIAS" ]; then
      if [ -f "$OZSH_ALIAS" ] && [ ! -L "$OZSH_ALIAS" ]; then
        cp -u $OZSH_ALIAS $RLOC_ALIAS
      elif [ -f "$BASH_ALIAS" ] && [ ! -L "$BASH_ALIAS" ]; then
        cp -u $BASH_ALIAS $RLOC_ALIAS
      fi
      if output=$(git -C $repo status --porcelain) && [ ! -z "$output" ]; then
        git -C $repo add .
        git -C $repo commit -m "Actualización"
      fi
      git -C $repo pushall
      unset output
      unset repo
      recargar
    else
      echo-error "no se pudo empujar al repositorio; no existe el archivo '$RLOC_ALIAS'"
    fi
    ;;
  enlazar | link)
    if [ -f "$RLOC_ALIAS" ]; then
      if [ -f "$OZSH_ALIAS" ]; then
        ln -fs $RLOC_ALIAS $OZSH_ALIAS
      fi
      if [ -f "$BASH_ALIAS" ]; then
        ln -fs $RLOC_ALIAS $BASH_ALIAS
      fi
      recargar
    else
      echo-error "no se encontró archivo '$RLOC_ALIAS' para copia."
    fi
    ;;
  -h | --help | help | ayuda)
    aliases-help ;;
  *)
    alias
    ;;
  esac
}

aliases-help () {
  echo "aliases: utilidad para hacer gestionar alias.

Uso:
  aliases [OPT]

Opciones:
  actualizar | update         Actualiza aliases.
  ver | see                   Muestra contenido de archivo de aliases.
  editar | edit               Edita aliases.
  empujar | push              Empuja los cambios locales al repositorio.
  enlazar | link              Genera enlace simbólico entre repo 'aliases' y archivo de aliases.
  -h | --help | help | ayuda  Despliega esta ayuda.
  vacío                       Uso regular de 'alias'.

Ejemplos:
  aliases ver
  aliases editar
  aliases -h
  aliases"
}

### Recarga aliases

recargar () {
  if [ -f "$OZSH_ALIAS" ]; then
    source $HOME/.zshrc
  else
    source $HOME/.bashrc
  fi
}

### Respalda discos

bak () {
  echo -n "¿Respaldar $1 en $2? [y/N] "
  read REPLY
  if [[ $REPLY =~ ^[Yy]$ ]]; then
    sudo dd if=$1 of=$2 bs=64K conv=noerror,sync status=progress
  else
    echo "No se realizó respaldo"
  fi
}

### Entra a descargas

cdd () {
  if [ -d "$HOME/Descargas" ]; then
    cd $HOME/Descargas
  else
    cd $HOME/Downloads
  fi
}

### Entra a nubes

nube () {
  if [ -d "$HOME/Nube" ]; then
    cd $HOME/Nube
  else
    cd $HOME/Cloud
  fi
}

## Instalador de fuentes

font-add () {
  if [ ! -d "$FONT_DIR" ]; then
    mkdir $FONT_DIR
  fi
  cp $@ $FONT_DIR
  fc-cache -fv
}

## Conversor de imágenes o documentos con ImageMagick o Pandoc

convertir () {
  echo "convertir FORMATO1 a FORMATO2 [con pandoc]"
  for file in *.$1; do
    echo "$file => $file.$3"
    if [ "$#" -eq 5 ]; then
      doc2md $file -o $file.$3
    else
      convert -flatten $file $file.$3
    fi
  done
}

## Gestiona archivos MD

md () {
  case $1 in
  formatear | format | f)
    pandoc --reference-links --reference-location=document \
      --columns=$LENGTH -o $2 $2 ;;
  convertir | convert)
    pandoc --markdown-headings=atx --standalone \
      -V lang=es -o $4 $2 ;;
  leer | read | ver | cat | c)
    glow -w $LENGTH -p $2 ;;
  editar | edit | e)
    $EDITOR $2 ;;
  -h | --help | help | ayuda)
    md-help ;;
  esac
}

md-help () {
  echo "md: utilidad para hacer gestionar archivos Markdown.

Uso:
  md [OPT]

Opciones:
  formatear | format | f      Da formato al archivo.
  convertir | convert         Convierte el archivo.
  leer | read | ver | cat | c Muestra el archivo.
  editar | edit | e           Edita el archivo con $EDITOR.
  -h | --help | help | ayuda  Despliega esta ayuda.

Ejemplos:
  md formatear archivo.md
  md convertir archivo.md a archivo.html
  md leer archivo.md
  md editar archivo.md
  md ayuda"
}

# Obtiene directorio padre

parent () {
  if [ $# -eq 0 ]; then curr="$PWD"; else curr=$1; fi
  echo $(python3 -c """
from pathlib import Path
print(Path('$curr').resolve().parent)
  """)
  unset curr
}

## Desactiva o activa virtualenv

check-venv () {
  if [ $# -eq 0 ]; then curr="$PWD"; else curr=$1; fi
  if [ -f "$curr/$VENV/bin/activate" ]; then
    source $curr/$VENV/bin/activate
  elif [[ $(parent $1) != "/" ]]; then
    check-venv $(parent $1)
  else
    type deactivate &>/dev/null && deactivate
  fi
  unset curr
}

check-venv # Ejecuta cuando se carga

## Agrega y arranca virtualenv

venv () {
  virtualenv $VENV
  check-venv
}

## Modifica comportamiento de cd

cd () {
  builtin cd $1
  check-venv
}

## Modifica comportamiento de git

git () {
  if [ "$#" -lt 3 ] && [ "$1" = "push" ] && ( [ -z "$2" ] || [ "$2" = "--tags" ] ); then
    /usr/bin/git pushall
    if [ "$2" = "--tags" ]; then
      /usr/bin/git pushtags
    fi
  else
    /usr/bin/git $@
  fi
}

## Embellece y verifica código

pymp () {
  case $1 in
  -f | formatear | format)
    black --line-length $LENGTH ${@:$#} ;;
  -h | --help | help | ayuda)
    pymp-help ;;
  esac
  if [ ! -d "${@:$#}" ] && [ ! -f "${@:$#}" ]; then
    echo-error "último argumento debe ser un fichero existente."
  fi
  flake8 --max-line-length $LENGTH ${@:$#}
  vulture ${@:$#}
}

pymp-help () {
  echo "pymp: utilidad para enchular código de Python.

Uso:
  pymp [OPT] [FILE|DIR]

Opciones:
  -f | formatear | format     Da formato a los archivos.
  -h | --help | help | ayuda  Despliega esta ayuda.

Ejemplos:
  pymp archivo.py
  pymp formatear .
  pymp ayuda"
  return 0
}
